#version 300 es
precision highp float;

struct DirectionalLight {
  vec3 color;
  float ambientIntensity;
  vec3 direction;
  float diffuseIntensity;
};

struct Material{
  float specularIntensity;
  float shininess;
  sampler2D colorTexture;
};

uniform DirectionalLight light;
uniform Material material;
uniform vec3 cameraEyePosition;

layout(location=0) out vec4 outColor;

in vec3 fragNormal;
in vec2 texUV;
in vec3 vertexWorldPosition;

vec3 calcDiffuse(){
  float factor = dot(light.direction,normalize(fragNormal));
  factor = max(factor,0.0);
  return light.color * light.diffuseIntensity * factor;
}
vec3 calcSpecular(){
  vec3 v = normalize(cameraEyePosition - vertexWorldPosition);
  vec3 n = 2.0 * normalize(fragNormal) *dot(fragNormal,light.direction) - light.direction;
  float cosineRV = max(dot(v,n),0.0);
  return pow(cosineRV, material.shininess) * light.color * material.specularIntensity;
}

void main() {
  vec3 lightAmbient = light.color * light.ambientIntensity;
  vec3 lightDiffuse = calcDiffuse();
  vec3 lightSpecular = calcSpecular();
  vec4 materialColor = texture(material.colorTexture,texUV);
  materialColor *= vec4(lightAmbient + lightDiffuse + lightSpecular,1.0);
  outColor = vec4(materialColor.xyz,1.0);
}